<?php
/**
* This is a class for formatting the results of the diff-class into
* a Unix `diff' compatible unified- or context-diff.
* 
* 
* This class is based on the Perl Module Algorithm::Diff, which was 
* written by Ned Konz and Mark-Jason Dominus. Also the documentation of 
* this module was written by them.
* Thank you two :)
* 
*/
class formatdiff
{
	var $a=array();
	var $b=array();
	var $acount=0;
	var $bcount=0;
	
	var $diff=array();
	var $type="u";
	var $context=3;
	
	
	/**
	* constructor of this module.
	* 
	* @param array a
	* @param array b
	* @param array diff
	* @param string type=u
	* @param integer context=3
	*/
	function formatdiff(&$a, &$b, $diff,$type="u",$context=3)
	{
		$this->a=&$a;
		$this->b=&$b;
		$this->acount=count($this->a);
		$this->bcount=count($this->b);
		$this->diff=$diff;
		
		// verify diff types
		$type=strtolower($type);
		switch($type)
		{
			case "u": break;
			case "c": break;
			default: $type="u"; // unified diff per default
		}
		
		// verify context lines (minimum of 3)
		$context=intval($context);
		if($context<3) $context=3;
		$this->context=$context;
		$this->type=$type;
	}
	
	
	/**
	* each hunk will generate its own output. 
	* Loop over hunks. If a hunk overlaps with the last hunk, join them.
	* Otherwise, print out the old one.
	*/
	function output()
	{
		$output="";
		$File_Length_Difference=0;
		
	
		$output.=(($this->type=="u") ? ("---") : ("***"))." file1	".date("D M m H:i:s Y")."\n";
		$output.=(($this->type=="u") ? ("+++") : ("---"))." file2	".date("D M m H:i:s Y")."\n";
		
		$oldhunk=null;
		foreach($this->diff as $piece)
		{
			$hunk = new Hunk($this->a, $this->b, $this->acount, $this->bcount, $piece, $this->context, $File_Length_Difference);
			if(!$oldhunk)
			{
				$oldhunk=$hunk;
				continue;
			}
			
			// if two neighboor hunks overlap, the current hunk is 
			// prepended to the old one.
			if($hunk->does_overlap($oldhunk))
			{
				$hunk->prepend_hunk($oldhunk);
			}
			// oldhunk
			else $output.=$oldhunk->output_diff($this->type);
			
			$oldhunk=$hunk;
		}
		// print out last hunk
		if($hunk) $output.=$hunk->output_diff($this->type);
		return $output;
	}
}










/**
* in a diff with context, each hunk contains at least one block
* of changes/inserts/removes..
*/
class Block
{
	var $changes=array();
	var $length_diff=0;
	var $removes=null;
	var $inserts=null;
	
	/**
	* Input is a chunk from &Algorithm::LCS::diff
	* Fields in a block:
	* length_diff - how much longer file 2 is than file 1 due to this block
	* Each change has:
	* sign        - '+' for insert, '-' for remove
	* item_no     - number of the item in the file (e.g., line number)
	* We don't bother storing the text of the item
	*/
	function Block($chunk)
	{
		$this->changes=array();
		$this->removes=null;
		$this->inserts=null;
		
		$this->changes=$chunk;
		$this->length_diff = count($this->insert()) - count($this->remove());
	}
	
	
	function op()
	{
		//what kind of block is this?
		$insert = $this->insert();
		$remove = $this->remove();
		if($remove && $insert) return '!';
		elseif($remove) return '-';
		elseif($insert) return '+';
		else return '^'; // context block
	}
	
	/**
	* Returns a list of the changes in this block that remove items
	*/
	function remove()
	{
		if(is_null($this->removes))
		{
			$this->removes = array();
			foreach($this->changes as $item)
			{
				if($item['sign']=="-") $this->removes[]=$item;	
			}
		}
		return $this->removes;
	}
	
	/**
	* Returns a list of the changes in this block that insert items
	*/
	function insert()
	{
		if(is_null($this->inserts))
		{
			$this->inserts = array();
			foreach($this->changes as $item)
			{
				if($item['sign']=="+") $this->inserts[]=$item;	
			}
		}
		return $this->inserts;
	}


}




/**
* A Hunk is a group of Blocks which overlap because of the
* context surrounding each block. (So if we're not using context, every
* hunk will contain one block.)
*/
class Hunk
{
	var $a;
	var $b;
	var $acount;
	var $bcount;
	var $start1;
	var $start2;
	var $end1;
	var $end2;
	var $blocks=array();
	
	
	
	/**
	* --
	*/
	function Hunk(&$a, &$b, $acount, $bcount, $piece, $context_items, &$File_Length_Difference)
	{
		$this->a=&$a;
		$this->b=&$b;
		$this->acount=$acount;
		$this->bcount=$bcount;
		
		$block = new Block ($piece); // this modifies $FLD!
		$before_diff = $File_Length_Difference; // BEFORE this hunk
		$after_diff = $before_diff + $block->length_diff;
		$File_Length_Difference += $block->length_diff;
		
		// $remove_array and $insert_array hold the items to insert and remove
		// Save the start & beginning of each array. If the array doesn't exist
		// though (e.g., we're only adding items in this block), then figure
		// out the line number based on the line number of the other file and
		// the current difference in file lenghts
		$remove_array = $block->remove();
		$insert_array = $block->insert();
		
		$a1 = $remove_array ? $remove_array[0]['item_no'] : -1;
		end($remove_array);
		$a2 = $remove_array ? $remove_array[key($remove_array)]['item_no'] : -1;
		
		$b1 = $insert_array ? $insert_array[0]['item_no'] : -1;
		end($insert_array);
		$b2 = $insert_array ? $insert_array[key($insert_array)]['item_no'] : -1;

		$start1 = $a1 == -1 ? $b1 - $before_diff : $a1;
		$end1   = $a2 == -1 ? $b2 - $after_diff  : $a2;
		$start2 = $b1 == -1 ? $a1 + $before_diff : $b1;
		$end2   = $b2 == -1 ? $a2 + $after_diff  : $b2;



		// At first, a hunk will have just one Block in it
	        $this->start1=$start1;
	        $this->start2=$start2;
	        $this->end1=$end1;
	        $this->end2=$end2;
	        $this->blocks=array($block);
	        
		$this->flag_context($context_items);

	}
	
	/**
	* Change the "start" and "end" fields to note that context should be added
	* to this hunk
	*/
	function flag_context($context_items)
	{
		if(!$context_items) return false; // no context

		// add context before
		$start1 = $this->start1;
		$num_added = (($context_items > $start1) ? ($start1) : ($context_items));
		$this->start1 -= $num_added;
		$this->start2 -= $num_added;
		
		// add context after
		$end1 = $this->end1;
		$num_added = (($end1+$context_items > $this->acount-1) ? ($this->acount - 1 - $end1) : ($context_items));
		$this->end1 += $num_added;
		$this->end2 += $num_added;
	}	

	
	/**
	* Is there an overlap between this hunk and the old one?
	* Note: if end of old hunk is one less than beginning of second, they overlap
	*/
	function does_overlap($oldhunk)
	{
		// first time through, $oldhunk is empty
		if(!$oldhunk || !is_object($oldhunk)) return false;
		
		// Do I actually need to test both?
		if(($this->start1 - $oldhunk->end1 <= 1) || ($this->start2 - $oldhunk->end2 <= 1)) return true;
		else return false;
	}
	
	/** 
	* Prepend hunk $oldhunk to the current hunk
	* Note that the old hunk isn't updated! Only this one is.
	*/
	function prepend_hunk($oldhunk)
	{
		$this->start1 = $oldhunk->start1;
		$this->start2 = $oldhunk->start2;
		
		$oldhunk->blocks = array_reverse($oldhunk->blocks);
		foreach($oldhunk->blocks as $block) array_unshift($this->blocks, $block);
	}
	
	
	/**
	* DIFF OUTPUT ROUTINES. THESE ROUTINES CONTAIN DIFF FORMATTING INFO...
	*/
	function output_diff($type)
	{
		if($type=="u") return $this->output_unified_diff();
		elseif($type=="c") return $this->output_context_diff();
		else die("unknown diff");
	}
	
	/**
	* output unified diff
	*/
	function output_unified_diff()
	{
		global $benchmark;
		$output="";
		
		// Calculate item number range.
		$range1 = $this->unified_range(1);
		$range2 = $this->unified_range(2);
		$output .= "@@ -$range1 +$range2 @@\n";
		
		// Outlist starts containing the hunk of file 1.
		// Removing an item just means putting a '-' in front of it.
		// Inserting an item requires getting it from file2 and splicing it in.
		//    We splice in $num_added items. Remove blocks use $num_added because
		// splicing changed the length of outlist.
		//    We remove $num_removed items. Insert blocks use $num_removed because
		// their item numbers---corresponding to positions in file *2*--- don't take
		// removed items into account.
		$low = $this->start1;
		$hi = $this->end1;
		$num_added = $num_removed = 0;
		$outlist = array();
		
		for($i=$low; $i<=$hi; $i++) $outlist[]=" ".$this->a[$i]; // assume it's just context

		foreach ($this->blocks as $block)
		{
			// removes
			foreach ($block->remove() as $item)
			{
				$offset = $item['item_no'] - $low + $num_added;
				$outlist[$offset]=$item['sign'].substr($outlist[$offset],1);
				$num_removed++;
			}
	
			// inserts
			$insert=array();
			unset($offset);
			foreach ($block->insert() as $item)
			{
				if(!isset($offset)) $offset = $item['item_no'] - $this->start2 + $num_removed;
				$insert[]=$item['sign'].$this->b[$item['item_no']];
				$num_added++;
			}
			// splice inserts into outlist
			array_splice($outlist, $offset, 0, $insert);
		}

		$output.=implode("\n",$outlist)."\n";
		return $output;
	}



	/**
	* Generate a range of item numbers to print for unified diff
	* Print number where block starts, followed by number of lines in the block
	* (don't print number of lines if it's 1)
	*/
	function unified_range($flag)
	{
		$start = $this->{"start$flag"};
		$end = $this->{"end$flag"};
		// index from 1, not zero
		$start++;
		$end++;
		$length = $end - $start + 1;
		// strange, but correct...
		$first = ($length < 2) ? ($end) : ($start); 
		$range = ($length == 1) ? ($first) : ("$first,$length");
		return $range;
	}
	
	
	
	
	
	
	/**
	* output context diff
	*/
	function output_context_diff()
	{
		$output = "***************\n";
		// Calculate item number range.
		$range1 = $this->context_range(1);
		$range2 = $this->context_range(2);
		
		// Print out file 1 part for each block in context diff format if there are
		// any blocks that remove items
		$output .= "*** $range1 ****\n";
		$low = $this->start1;
		$hi  = $this->end1;
		
		$blocklist=array();
		foreach($this->blocks as $block)
		{
			if($block->remove()) $blocklist[]=$block;
		}
		if($blocklist)
		{
			$outlist=array();
			for($i=$low; $i<=$hi; $i++) $outlist[]=" ".$this->a[$i]; // assume it's just context
			foreach($blocklist as $block)
			{
				$op = $block->op(); // - or !
				foreach ($block->remove() as $item)
				{
					$outlist[$item['item_no'] - $low]=$op.substr($outlist[$item['item_no'] - $low],1);
				}
			}
			$output.=implode("\n",$outlist)."\n";
		}

		
		$output .= "--- $range2 ----\n";
		$low = $this->start2;
		$hi  = $this->end2;

		$blocklist=array();
		foreach($this->blocks as $block)
		{
			if($block->insert()) $blocklist[]=$block;
		}
		if($blocklist)
		{
			$outlist=array();
			for($i=$low; $i<=$hi; $i++) $outlist[]=" ".$this->b[$i]; // assume it's just context
			foreach ($blocklist as $block)
			{
				$op = $block->op(); // + or !
				foreach ($block->insert() as $item)
				{
					$outlist[$item['item_no'] - $low] = $op.substr($outlist[$item['item_no'] - $low],1);	
				}	
			}
		}
		$output .= implode("\n", $outlist)."\n";
		
		return $output;
	}





	/**
	* Generate a range of item numbers to print. 
	* Only print 1 number if the range
	* has only one item in it. Otherwise, it's 'start,end'
	*/
	function context_range($flag)
	{
		$start = $this->{"start$flag"};
		$end = $this->{"end$flag"};
		// index from 1, not zero
		$start++;
		$end++;
		$range = ($start < $end) ? "$start,$end" : $end;
		return $range;
	}
}
?>