<?php

/**
* session class, handles sessions and session variables
* only sessionvariables under 250 chars can be saved. 
*/

class session
{
	/**
	* @var string session id
	* @access private
	*/
	var $sid="";


	/**
	* @var array variables variables are stored in this array
	* @access private
	*/
	var $variables=array();


	/**
	* @var storetodb boolean true if there are new variables that must be stored
	* @access private
	*/
	var $storetodb=false;


	/**
	* @var boolean deletevars true if there variables where be deleted
	* @access private
	*/
	var $deletevars=false;


	/**
	* @var boolean cookie true if the client accepts cookies
	* @access private
	*/
	var $cookie=false;


	/**
	* @var boolean verifyip if this variable is true, the client's IP address must not change within a session
	* @access private
	*/
	var $verifyip=true;


	/**
	* @var boolean verifyua if this variable is true, the client's UserAgent/Browser must not change within a session
	* @access private
	*/
	var $verifyua=true;


	/**
	* @var integer maxlifetime this variable sets the maximal life time of a session
	* @access private
	*/
	var $maxlifetime=0;


	/**
	* @var string name this variable sets the name of a session
	* @access private
	*/
	var $name="sid";






	/**
	* constructor of this class, starts the current session
	*/
	function session($session_name="sid")
	{
		global $db, $_COOKIE, $_GET, $_POST;
		$this->verifyip=true;
		$this->verifyua=true;
		$this->maxlifetime=2*3600;
		$this->name($session_name);
		
		if(isSet($_COOKIE[($this->name)]) && $_COOKIE[($this->name)]!="") $sid=$_COOKIE[($this->name)];
		elseif(isSet($_POST[($this->name)]) && $_POST[($this->name)]!="") $sid=$_POST[($this->name)];
		elseif(isSet($_GET[($this->name)]) && $_GET[($this->name)]!="") $sid=$_GET[($this->name)];
		else $sid="";
		
		$ip=getIP();
		$ua=getUA();
		
		
		if($sid=="")
		{
			$session_info = $db->query_first("SELECT * FROM ".$db->getPrefix()."sessions WHERE ip='".addslashes($ip)."' AND lastactivity>='".(time()-($this->maxlifetime))."' AND ua='".addslashes($ua)."'",MYSQL_ASSOC);
			if($session_info['sid']!="")
			{
				$sid=$session_info['sid'];
				$createnew=false;
			}
			else $createnew=true;
		}
		else
		{
			$session_info = $db->query_first("SELECT * FROM ".$db->getPrefix()."sessions WHERE sid='".addslashes($sid)."' AND lastactivity>='".(time()-($this->maxlifetime))."'".ifelse($this->verifyip," AND ip='$ip'").ifelse($this->verifyua," AND ua='$ua'"),MYSQL_ASSOC);
			if($session_info['sid']!="") $createnew=false;
			else $createnew=true;
		}

		if(!$createnew && $session_info['sid']!="")
		{
			$this->sid=$sid;
			$lastactivity=time();
			if(isSet($_COOKIE[($this->name)]) && $_COOKIE[($this->name)]==$sid && $session_info['cookie']==0)
			{
				$session_info['cookie']=1;
				$db->unbuffered_query("UPDATE ".$db->getPrefix()."sessions SET lastactivity='$lastactivity', cookie='1' WHERE sid='".addslashes($sid)."'",1);
			}
			elseif((!isSet($_COOKIE[($this->name)]) || $_COOKIE[($this->name)]=="") && $session_info['cookie']==1)
			{
				$session_info['cookie']=0;
				$db->unbuffered_query("UPDATE ".$db->getPrefix()."sessions SET lastactivity='$lastactivity', cookie='0' WHERE sid='".addslashes($sid)."'",1);
			}
			else
			{
				$db->unbuffered_query("UPDATE ".$db->getPrefix()."sessions SET lastactivity='$lastactivity' WHERE sid='".addslashes($sid)."'",1);
			}
			if($session_info['cookie']==1) $this->cookie=true;
			else $this->cookie=false;
			$this->load();
		}
		elseif($createnew)
		{
			$sid=$this->create();
			$this->sid=$sid;
		}
		$this->gc();
	}
	







	/**
	* returns the session id - but only if the client does not accept cookies
	* 
	* @return string session id
	* @access public
	*/
	function sid()
	{
		if(!$this->cookie) return $this->sid;
		else return "";
	}










	/**
	* Get and/or set the current session name
	*
	* returns the name of the current session. If name is specified, 
	* the name of the current session is changed to its value. 
	*
	* @param string name
	* @return string name
	* @see session_name()
	* @access public
	*/
	function name($name="")
	{
		if($name!="") $this->name=$name;
		return $this->name;
	}









	/**
	* creates a new session
	*
	* @return string session id
	* @access private
	*/
	function create()
	{
		global $db;
		
		$this->variables=array();
		$this->storetodb=false;
		$this->sid=md5(uniqid(rand()));
		$this->cookie=false;
		
		
		$db->query("INSERT INTO ".$db->getPrefix()."sessions (sid,lastactivity,ip,ua,cookie) VALUES ('".addslashes($this->sid)."', '".time()."', '".addslashes(getIP())."', '".addslashes(getUA())."', '".((int)$this->cookie)."')");
		$db->unbuffered_query("DELETE FROM ".$db->getPrefix()."sessionvars WHERE sid='".addslashes($this->sid)."'",1);
		@setcookie($this->name,$this->sid,time()+$this->maxlifetime);
		
		return $this->sid;
	}





	/**
	* registers/stores a variable in the current session
	*
	* @param string variable
	* @param string value
	* @return void
	* @access public
	*/
	function register($variable,$value)
	{
		if(!($this->is_registered($variable)) || $this->get($variable)!=$value)
		{
			$this->storetodb=true;
			$this->variables[$variable]=$value;
		}
	}





	/**
	* deletes a variable in the current session
	*
	* @param string variable
	* @return void
	* @access public
	*/
	function unregister($variable)
	{
		if($this->is_registered($variable))
		{
			unset($this->variables[$variable]);
			$this->storetodb=true;
			$this->deletevars=true;
		}
	}




	/**
	* returns the value of a variable within the current session
	*
	* @param string variable
	* @return mixed value
	* @access public
	*/
	function get($variable)
	{
		if($this->is_registered($variable)) return $this->variables[$variable];
		else return false;
	}







	/**
	* returns true if variable is registered in the current session
	*
	* @param string variable
	* @return boolean
	* @see session_is_registered()
	* @access public
	*/
	function is_registered($variable)
	{
		if(isSet($this->variables[$variable])) return true;
		else return false;
	}








	/**
	* deletes all variables in the current session
	*
	* @return void
	* @see session_unset()
	* @access public
	*/
	function unset_all()
	{
		$this->variables=array();
		$this->storetodb=true;
		$this->deletevars=true;
	}








	/**
	* deletes all variables in the current session and destroys the current session.
	*
	* @return void
	* @see session_destroy()
	* @access public
	*/
	function destroy()
	{
		global $db;
		
		$this->unset_all();
		$this->storevariables();
		$db->unbuffered_query("DELETE FROM ".$db->getPrefix()."sessions WHERE sid='".$this->sid."'",1);
		
		@setcookie($this->name,"",time()-$this->maxlifetime);
	}







	/**
	* stores the current session variables in the database
	*
	* normally called when the script ends (see register_shutdown_function())
	*
	* @return void
	* @access public
	*/
	function store()
	{
		global $db;
		
		if($this->storetodb)
		{
			if($this->deletevars)
			{
				$db->unbuffered_query("DELETE FROM ".$db->getPrefix()."sessionvars WHERE sid='".$this->sid."'",1);
			}
			if(count($this->variables)>0)
			{
				$insertstr="";
				reset($this->variables);
				while(list($variable,$value)=each($this->variables))
				{
					$insertstr.=",('".addslashes($this->sid)."', '".addslashes($variable)."', '".addslashes($value)."')";
				}
				if($insertstr)
				{
					$db->unbuffered_query("REPLACE INTO ".$db->getPrefix()."sessionvars (sid,variable,value) VALUES ".substr($insertstr,1),1);
				}
			}
		}
	}









	/**
	* load the session variables from the database
	*
	* @return void
	* @access private
	*/
	function load()
	{
		global $db;
		
		$result = $db->query("SELECT variable,value FROM ".$db->getPrefix()."sessionvars WHERE sid='".addslashes($this->sid)."'");
		while($row=$db->fetch_array($result,MYSQL_ASSOC))
		{
			$this->variables[$row['variable']]=$row['value'];
		}
	}





	/**
	* garbage collector, deletes old session data.
	*
	* @return void
	* @access private
	*/
	function gc()
	{
		global $db;
		
		srand(date("s"));
		if(rand(1,60)==30)
		{
			$sids="";
			$result = $db->query("SELECT sid FROM ".$db->getPrefix()."sessions WHERE lastactivity<'".(time()-($this->maxlifetime))."' AND sid<>'".addslashes($this->sid)."'");
			while(list($sid)=$db->fetch_array($result,MYSQL_NUM)) $sids.=",'".addslashes($sid)."'";
			if($sids)
			{
				$db->unbuffered_query("DELETE FROM ".$db->getPrefix()."sessions WHERE sid IN (".substr($sids,1).")",1);
				$db->unbuffered_query("DELETE FROM ".$db->getPrefix()."sessionvars WHERE sid IN (".substr($sids,1).")",1);
				return true;
			}
			return false;
		}
		else return false;
	}
}
?>