<?php

class writetar
{
	var $archivename="";
	var $compression_level=-1;
	var $opened=0;
	var $filehandle="";
	
	function writetar($archivename,$compression_level=-1)
	{
		$this->archivename=$archivename;
		$this->opened=0;
		$this->compression_level=$compression_level;
		if(!extension_loaded("zlib")) $this->compression_level=-1;
	}
	
	function error($msg)
	{
		die("Fatal error: $msg");
	}
	function open()
	{
		if(!$this->opened)
		{
			if($this->compression_level>=0) $this->filehandle=@gzopen($this->archivename,"wb".$this->compression_level);
			else $this->filehandle=@fopen($this->archivename,"wb");
			if(!$this->filehandle) $this->error("cannot open ".$this->archivename." for write");
			$this->opened=1;
		}
	}
	function close()
	{
		if($this->opened==1)
		{
			if($this->compression_level>=0) @gzclose($this->filehandle);
			else @fclose($this->filehandle);
		}
	}
	
	function create()
	{
		//
	}
	
	function add_dir($dir, $date="")
	{
		// read file info
		$reduce_filename = $dir;
		$size = 0;
		
		$uid = sprintf("%6s ", DecOct(0));
		$gid = sprintf("%6s ", DecOct(0));
		$perms = sprintf("%6s ", DecOct(0));
		$mtime = sprintf("%11s", DecOct(time()));
		$typeflag = "5";
		$v_size = sprintf("%11s ", DecOct(0));

		$linkname = "";
		$magic = "";
		$version = "";
		$uname = "";
		$gname = "";
		$devmajor = "";
		$devminor = "";
		$prefix = "";
		
		// Compose the binary string of the header in two parts arround the checksum position
		$binary_data_first = pack("a100a8a8a8a12A12", $reduce_filename, $perms, $uid, $gid, $size, $mtime);
		$binary_data_last = pack("a1a100a6a2a32a32a8a8a155a12", $typeflag, $linkname, $magic, $version, $uname, $gname, $devmajor, $devminor, $prefix, "");

		// Calculate the checksum
		$checksum = 0;
		// First part of the header
		for($i=0; $i<148; $i++)
		{
			$checksum += ord(substr($binary_data_first,$i,1));
		}
		// Ignore the checksum value and replace it by ' ' (space)
		for ($i=148; $i<156; $i++)
		{
			$checksum += ord(' ');
		}
		// Last part of the header
		for ($i=156, $j=0; $i<512; $i++, $j++)
		{
			$checksum += ord(substr($binary_data_last,$j,1));
		}
		
		// Write the first 148 bytes of the header in the archive
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data_first, 148);
		else fwrite($this->filehandle, $binary_data_first, 148);
		
		// Write the calculated checksum
		$checksum = sprintf("%6s ", DecOct($checksum));
		$binary_data = pack("a8", $checksum);
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data, 8);
		else fwrite($this->filehandle, $binary_data, 8);
		
		// Write the last 356 bytes of the header in the archive
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data_last, 356);
		else fwrite($this->filehandle, $binary_data_last, 356);
	}
	
	
	
	
	function add_file($src, $filename, $date="")
	{
		// read file info
		$reduce_filename = $filename;

		$file_info = stat($src);
		$uid = sprintf("%6s ", DecOct($file_info[4]));
		$gid = sprintf("%6s ", DecOct($file_info[5]));
		$perms = sprintf("%6s ", DecOct(fileperms($src)));
		$mtime = sprintf("%11s", DecOct(filemtime($src)));
		$typeflag = "";
		$size = sprintf("%11s ", DecOct($file_info[7]));

		$linkname = "";
		$magic = "";
		$version = "";
		$uname = "";
		$gname = "";
		$devmajor = "";
		$devminor = "";
		$prefix = "";
		
		// Compose the binary string of the header in two parts arround the checksum position
		$binary_data_first = pack("a100a8a8a8a12A12", $reduce_filename, $perms, $uid, $gid, $size, $mtime);
		$binary_data_last = pack("a1a100a6a2a32a32a8a8a155a12", $typeflag, $linkname, $magic, $version, $uname, $gname, $devmajor, $devminor, $prefix, "");

		// Calculate the checksum
		$checksum = 0;
		// First part of the header
		for($i=0; $i<148; $i++)
		{
			$checksum += ord(substr($binary_data_first,$i,1));
		}
		// Ignore the checksum value and replace it by ' ' (space)
		for ($i=148; $i<156; $i++)
		{
			$checksum += ord(' ');
		}
		// Last part of the header
		for ($i=156, $j=0; $i<512; $i++, $j++)
		{
			$checksum += ord(substr($binary_data_last,$j,1));
		}
		
		// Write the first 148 bytes of the header in the archive
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data_first, 148);
		else fwrite($this->filehandle, $binary_data_first, 148);
		
		// Write the calculated checksum
		$checksum = sprintf("%6s ", DecOct($checksum));
		$binary_data = pack("a8", $checksum);
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data, 8);
		else fwrite($this->filehandle, $binary_data, 8);
		
		// Write the last 356 bytes of the header in the archive
		if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data_last, 356);
		else fwrite($this->filehandle, $binary_data_last, 356);
		
		
		
		// read file by 512 octet blocks
		$fh = @fopen($src,"rb");
		$i=0;
		while (($buffer = fread($fh, 512)) != "")
		{
			$binary_data = pack("a512", "$buffer");
			if($this->compression_level>=0) gzwrite($this->filehandle, $binary_data);
			else fwrite($this->filehandle, $binary_data);
			$i++;
		}
		fclose($fh);
	}
	
	
	function add_tree($dir)
	{
		$dir=str_replace("\\","/",$dir);
		if(substr($dir,-1)=="/") $dir=substr($dir,0,strlen($dir)-1);
		if(!file_exists($dir)) $this->error("directory $dir does not exist.");
		
		$dirs=array();
		$files=array();
		$this->get_tree($dir,$dirs,$files);
		
		if(count($dirs) || count($files))
		{
			foreach($dirs as $dirname)
			{
				if($dirname!=$dir && strlen($dirname)>strlen($dir))
				{
					#echo "Adding directory ".substr($dirname,strlen($dir)+1)."<br>";
					$this->add_dir(substr($dirname,strlen($dir)+1));
				}
			}
			foreach($files as $filename)
			{
				#echo "Adding file ".substr($filename,strlen($dir)+1)."<br>";
				$this->add_file($filename,substr($filename,strlen($dir)+1));
			}
		}
		return $files;
	}
	
	function get_tree($dir,&$dirs,&$files)
	{
		$dirs[]=$dir;
		$dirh=@opendir($dir);
		if(!$dirh) $this->error("Cannot open directory $dir for reading.");
		while($filename=readdir($dirh))
		{
			if($filename=="." || $filename=="..") continue;
			elseif(is_dir($dir."/".$filename)) $this->get_tree($dir."/".$filename,$dirs,$files);
			elseif(is_file($dir."/".$filename)) $files[]=$dir."/".$filename;
		}
		@closedir($dirh);
	}
}






class readtar
{
	var $archivename="";
	var $opened=0;
	var $filehandle="";
	var $contentlist=array();
	var $read=0;
	
	function readtar($archivename)
	{
		if(!file_exists($archivename)) $this->error("archive $archivename does not exist");
		$this->archivename=$archivename;
		$this->open();
		$this->readcontent();
		$this->close();
	}
	function open()
	{
		if($this->opened==0)
		{
			$this->filehandle=@fopen($this->archivename,"rb");
			if(!$this->filehandle) $this->error("cannot open archive ".$this->archivename." for reading");
			$this->opened=1;
		}
	}
	function close()
	{
		if($this->opened==1)
		{
			@fclose($this->filehandle);
			$this->opened=0;
		}
	}
	function error($msg)
	{
		die("Fatal Error: $msg");
	}
	
	function readcontent()
	{
		$this->contentlist=array();
		$this->read=1;
		
		$this->open();
		
		 // ----- Read the blocks
		#While(!($eof = (@feof($this->filehandle))))
		$i=0;
		while(!feof($this->filehandle))
		{
			// Read the 512 bytes header
			$binary_data = fread($this->filehandle, 512);
			
			// read header
			$header = $this->readheader($binary_data);
			if(is_array($header))
			{
				$this->contentlist[$i]=$header;
				$this->contentlist[$i]['index']=$i;
				$i++;
			}
		}
		$this->close();
	}
	
	
	function readheader($binary_data)
	{
		$header=array();
		
		if(strlen($binary_data)==512)
		{
			// Calculate the checksum
			$checksum = 0;
			// First part of the header
			for ($i=0; $i<148; $i++)
			{
				$checksum+=ord(substr($binary_data,$i,1));
			}
			// Ignore the checksum value and replace it by ' ' (space)
			for ($i=148; $i<156; $i++)
			{
				$checksum += ord(' ');
			}
			// Last part of the header
			for ($i=156; $i<512; $i++)
			{
				$checksum+=ord(substr($binary_data,$i,1));
			}
			
			// Extract the values
			$data = unpack("a100filename/a8mode/a8uid/a8gid/a12size/a12mtime/a8checksum/a1typeflag/a100link/a6magic/a2version/a32uname/a32gname/a8devmajor/a8devminor", $binary_data);
			
		    // ----- Extract the properties
			$header['checksum'] = OctDec(trim($data['checksum']));
			if($header['checksum']==$checksum)
			{
				$header['filename'] = trim($data['filename']);
				$header['mode'] = OctDec(trim($data['mode']));
				$header['uid'] = OctDec(trim($data['uid']));
				$header['gid'] = OctDec(trim($data['gid']));
				$header['size'] = OctDec(trim($data['size']));
				$header['mtime'] = OctDec(trim($data['mtime']));
				if(($header['typeflag'] = $data['typeflag']) == "5")
				{
					$header['size'] = 0;
					$header['type']="folder";
				}
				else $header['type']="file";
				$header['offset']=ftell($this->filehandle);
				return $header;
			}
			else return false;
		}
		else return false;
	}
	
	
	function contentlist()
	{
		if(!$this->read!=1) $this->readcontent();
		return $this->contentlist;
	}
	
	function extract($index,$destination)
	{
		if($this->read!=1) $this->readcontent();
		$this->open();
		if(isset($this->contentlist[$index])) $header = $this->contentlist[$index];
		else $this->error("cannot find file $index in archive");
		if($header['type']!="file") $this->error("cannot extract a folder");
		
		// seek to offset
		fseek($this->filehandle,$header['offset']);
		
		$fh = @fopen($destination, "wb");
		if(!$fh) $this->error("cannot open $destination for writing");

		// read data
		$n = floor($header['size']/512);
		for($i=0; $i<$n; $i++)
		{
			$content = fread($this->filehandle, 512);
			fwrite($fh, $content, 512);
		}
		if(($header['size'] % 512) != 0)
		{
			$content = fread($this->filehandle, 512);
			fwrite($fh, $content, ($header['size'] % 512));
		}
		fclose($fh);
		@chmod($destination,0777);
		@touch($destination, $header['mtime']);
		
		
		$this->close();
	}
}

?>