<?php

/*
	Datei: class.upload.php
	Letzte nderung: 14.05.02
	Beschreibung: Klasse fr den einfachen Dateiupload
	mit berprfung des MIME-Types und der Dateierweiterung.
*/


if (!function_exists("is_uploaded_file") and get_cfg_var("safe_mode")==0) {
	function is_uploaded_file($filename) {
    		if (!$tmp_file = get_cfg_var('upload_tmp_dir')) $tmp_file = dirname(tempnam('', ''));
    		$tmp_file .= '/' . basename($filename);
    		return (ereg_replace('/+', '/', $tmp_file) == $filename);
  	} 
	function move_uploaded_file($filename, $destination) {
     		if (is_uploaded_file($filename))  {
       			if (copy($filename,$destination)) return true;
       			else return false;
       		}
      		else return false;
   	}
}



class upload
{
	var $mime_types = array(); // Erlaubte MIME Typen
	var $extensions = array(); // Erlaubte Dateierweiterungen
	var $uploaddir = ""; // Uploadverzeichnis
	
	/*
		Methode: upload();
		Konstruktor der Klasse, setzt MIME Typen, Dateierweiterungen und Uploadverzeichnis
	*/
	function upload($mime_types=array("image/gif","image/jpg","image/pjpeg","image/x-png","text/plain","text/html","application/x-zip-compressed"),$extensions=array("gif","jpg","jpeg","png","txt","zip"),$dir=".")
	{
		$this->mime_types = $mime_types;
		$this->extensions = $extensions;
		$this->uploaddir = $dir;
	}
	
	/*
		Methode: setUploadDir();
		setUploadDir() setzt nachtrglich ein neues Uploadverzeichnis
	*/
	function setUploadDir($dir)
	{
		$this->uploaddir = $dir;
		return true;
	}
	
	/*
		Methode: checkmime()
		checkmime() berprft den MIME Type des Uploads
		und gibt bei zulssigem MIME Type true zurck, ansonsten false.
	*/
	function checkmime($file)
	{
		global $_FILES;
		if(!in_array($_FILES[$file]['type'],$this->mime_types))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	/*
		Methode: checkextension()
		checkextension() berprft die Dateierweiterung des Uploads
		und gibt bei zulssiger Endung true zurck, ansonsten false.
	*/
	function checkextension($file)
	{
		if(!in_array($this->extract_extension($file),$this->extensions))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	
	/*
		Methode: extract_extension()
		extract_extension() ermittelt die Dateierweiterung des Uploads
		und gibt sie zurck.
	*/
	function extract_extension($file)
	{
		global $_FILES;
		return get_fileextension($_FILES[$file]['name']);
	}

	
		/*
		Methode: extract_filename()
		extract_filename() ermittelt den Dateinamen des Uploads
		und gibt ihn zurck.
	*/
	function extract_filename($file)
	{
		global $_FILES;
		return get_filename($_FILES[$file]['name']);
	}


	
	/*
		Methode: doUpload()
		doUpload() kopiert den Upload in die in $destination angegebene Datei
		und gibt bei Erfolg true zurck, ansonsten false.
	*/
	function doUpload($file,$destination)
	{
		global $_FILES;
		if(move_uploaded_file($_FILES[$file]['tmp_name'],($this->uploaddir."/".$destination)))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

}





/*
	Verwendungsbeispiel:

HTML Formular:
==========================
<html>
<body>
<form action="upload.php" method="post" enctype="multipart/form-data">
<input type="hidden" name="MAX_FILE_SIZE" value=\"20000000\">
<input type="file" name="fileupload1">
<input type="hidden" name="send" value="send">
<input type="submit" value="Datei Hochladen">
</body>
</html>
==========================

Datei upload.php
==========================
$upload = new upload();
$upload->setUploadDir(".");
$file = "fileupload1";

if(!$upload->checkmime($file) || !$upload->checkextension($file)) echo "Falscher MIME TYPE / EXTENSION (".$_FILES[$file][type]." - ".$_FILES[$file][name].")";
else {
	$done = $upload->doUpload($file,$_FILES[$file][name]);
	if($done) echo "Datei erfolgreich hochgeladen.";
	else echo "Fehler beim Dateiupload.";
}
==========================
*/



?>