<?php

/**
* returns the client's User Agent (Browser)
* 
* @param integer length
* @return string useragent
*/
function getUA($length=255)
{
	global $_SERVER;
	return substr($_SERVER['HTTP_USER_AGENT'],0,$length);
}







/**
* returns the client's IP address
* 
* @return string ip address
*/
function getIP()
{
	global $_SERVER;
	return $_SERVER['REMOTE_ADDR'];
}





/**
* calls the sessions' method to store the session variables into the database
* 
* @return void
*/
function session_store()
{
	global $session;
	$session->store();
}






/**
* fills the new superglobal arrays with the data of the old array
* 
* this function is only called if the phpversion is under 4.10
* the new arrays must be declared as global because they are not 
* superglobal in phpversions under 4.10.
*
* @return void
*/
function get_vars_old()
{
	global $HTTP_COOKIE_VARS, $HTTP_POST_FILES, $HTTP_POST_VARS, $HTTP_GET_VARS, $HTTP_SERVER_VARS,$HTTP_ENV_VARS, $HTTP_SESSION_VARS, $_REQUEST, $_COOKIE, $_POST, $_GET, $_SERVER, $_FILES,$_ENV,$_SESSION;
	
	if(is_array($HTTP_COOKIE_VARS))
	{
		while(list($key,$val)=each($HTTP_COOKIE_VARS))
		{
			$_REQUEST[$key]=$val;
			$_COOKIE[$key]=$val;
		}
	}

	if(is_array($HTTP_POST_VARS))
	{
		while(list($key,$val)=each($HTTP_POST_VARS))
		{
			$_REQUEST[$key]=$val;
			$_POST[$key]=$val;
		}
	}

	if(is_array($HTTP_GET_VARS))
	{
		while(list($key,$val)=each($HTTP_GET_VARS))
		{
			$_REQUEST[$key]=$val;
			$_GET[$key]=$val;
		}
	}
	
	if(is_array($HTTP_POST_FILES))
	{
		while(list($key,$val)=each($HTTP_POST_FILES)){
			$_FILES[$key]=$val;
		}
	}
	
	if(is_array($HTTP_SERVER_VARS))
	{
		while(list($key,$val)=each($HTTP_SERVER_VARS))
		{
			$_SERVER[$key]=$val;
		}
	}
	
	if(is_array($HTTP_ENV_VARS))
	{
		while(list($key,$val)=each($HTTP_ENV_VARS))
		{
			$_ENV[$key]=$val;
		}
	}
	
	if(is_array($HTTP_SESSION_VARS))
	{
		while(list($key,$val)=each($HTTP_SESSION_VARS))
		{
			$_SESSION[$key]=$val;
		}
	}
}






/**
* applies the phpfunction stripslashes() recursively on all elements of an array
* 
* @param array the array on which stripslashes() is applied.
* @return void
*
* @see trim_array()
*/
function stripslashes_array(&$array)
{
	reset($array);
	while(list($key,$val)=each($array))
	{
		if(is_string($val))
		{
			$array[$key]=stripslashes($val);
		}
		elseif(is_array($val))
		{
			// recursive function call
			$array[$key]=stripslashes_array($val);
		}
	}
	return $array;
}




/**
* applies the phpfunction trim() recursively on all elements of an array
* 
* @param array the array on which trim() is applied.
* @return void
*
* @see stripslashes_array()
*/
function trim_array(&$array)
{
	reset($array);
	while(list($key,$val)=each($array))
	{
		if(is_array($val))
		{
			// recursive function call
			$array[$key]=trim_array($val);
		}
  		elseif(is_string($val))
		{
			$array[$key]=trim($val);
		}
	}
	return $array;
}






/**
* encrypts a password using a one-way-hash function (in this case MD5)
* 
* This function is an alias to the php function md5()
*
* @param array the array on which stripslashes() is applied.
* @return string the hashed password
*/
function encrypt_password($password)
{
	return md5($password);
}







/**
* returns $returntrue if the condition is true and $returnfalse if the condition is false
* 
* this function is an alias to ($condition ? $returntrue : $returnfalse)
* 
* @param mixed condition
* @param mixed returntrue
* @param mixed optionally returnfalse
* @return mixed the returned value is $returntrue or $returnfalse
*/
function ifelse($condition,$returntrue,$returnfalse="")
{
	if($condition)
	{
		return $returntrue;
	}
	else
	{
		return $returnfalse;
	}
}










/**
* creates a (compressed) snapshot of an project at a certain timestamp
* 
* create_package() selects all directories/files of the project at the given timestamp
* and create a (compressed) archive (ZIP Archive or gzip-compressed Tarball).
* 
* @param integer project_id of the project
* @param string type of the snapshot
* @param integer timestamp
* @param string comment
* @param integer compression_level
* @param string archive_type
* @return integer snapshot_id of the created snapshot
*/
function create_package($project_id,$type,$timestamp,$comment,$compression_level,$archive_type)
{
	global $db;
	
	
	// lock project
	lock_project($project_id);
	
	if($type=="snapshots")
	{
		$db->query("INSERT INTO ".$db->getPrefix()."snapshots (project_id,createdate,timestamp,comment, compression_level, archive_type) VALUES('$project_id','".time()."','$timestamp','".addslashes($comment)."', '$compression_level', '$archive_type')");
		$snapshot_id = $db->insert_id();
	}
	
	if(!extension_loaded("zlib")) $compression_level=-1;
	if($compression_level==-1 && $archive_type=="tgz") $extension="tar";
	elseif($compression_level>=0 && $archive_type=="tgz") $extension="tar.gz";
	elseif($archive_type="zip") $extension="zip";
	
	if($archive_type=="zip")
	{
		include("class.zip.php");
		$archive = new writezip("./".$type."/".$snapshot_id.".".$extension, $compression_level);
	}
	else
	{
		include("class.tar.php");
		$archive = new writetar("./".$type."/".$snapshot_id.".".$extension, $compression_level);
	}
	$archive->open();
	
	$files = $db->query("SELECT f.dir,f.filename,f.extension,f.createdate as f_createdate,MAX(v.version_id) as version_id,MAX(v.createdate) as createdate,f.isfile 
	FROM ".$db->getPrefix()."files f LEFT OUTER JOIN ".$db->getPrefix()."versions v ON v.file_id=f.file_id 
	WHERE (f.project_id=$project_id) AND ((v.createdate IS NULL OR v.createdate<=$timestamp) AND f.createdate<=$timestamp) 
	AND (f.deletedate=0 OR f.deletedate>$timestamp) AND (v.deletedate IS NULL OR v.deletedate=0 OR v.deletedate>$timestamp) 
	GROUP BY f.file_id ORDER BY v.createdate DESC");

	while($file=$db->fetch_array($files,MYSQL_ASSOC))
	{
		$desiredpath = substr($file['dir'],1);
		$path = "";
		// folder
		if($file['isfile']==0)
		{
			$archive->add_dir(substr($file['dir'],1)."/",$file['f_createdate']);
		}
		// file
		elseif($file['isfile']==1)
		{
			$archive->add_file("./files/".$file['version_id'].ifelse($file['extension'],".".$file['extension']), ifelse($file['dir'],substr($file['dir'],1)."/").$file['filename'].ifelse($file['extension'],".".$file['extension']), $file['createdate']);
		}
	}

	$archive->create();
	$archive->close();
	$db->query("UPDATE ".$db->getPrefix()."snapshots SET checksum='".checksum_file("./".$type."/".$snapshot_id.".".$extension)."', size='".filesize("./".$type."/".$snapshot_id.".".$extension)."' WHERE snapshot_id='$snapshot_id'");

	unlock_project($project_id);
	return $snapshot_id;
}




/**
* locks a project for read and write
*
* @param integer project_id to lock
* @return void
*/
function lock_project($project_id)
{
	global $db;
	$db->query("UPDATE ".$db->getPrefix()."projects SET writelock=1 WHERE project_id=$project_id");
}


/**
* unlocks a locked project
*
* @param integer project_id to unlock
* @return void
*/
function unlock_project($project_id)
{
	global $db;
	$db->query("UPDATE ".$db->getPrefix()."projects SET writelock=0 WHERE project_id=$project_id");
}







/**
* deletes folder in a project
*
* subdirectories and files will be deleted too.
* files and directories are not deleted finally but are backuped.
*
* @param integer project_id
* @param array folders to delete
* @param array files to delete
* @param integer timestamp of the deletedate
* @return void
*/
function tmpdeletefolder($project_id,$deletefolder,&$deletefiles,$deletedate)
{
	global $db;
	foreach($deletefolder as $folder)
	{
		$sub = $db->query("SELECT file_id,dir,isfile FROM ".$db->getPrefix()."files WHERE project_id='$project_id' AND dir like '".addslashes($folder)."%'");
		while($row = $db->fetch_array($sub,MYSQL_ASSOC))
		{
			if($row['isfile'])
			{
				$deletefiles[] = $row['file_id'];
			}
			else
			{
				$deletefolder[] = $row['dir'];
			}
		}
	}
	$db->query("UPDATE ".$db->getPrefix()."files SET deletedate=$deletedate WHERE project_id='$project_id' AND dir IN('".implode("','",$deletefolder)."') AND isfile=0");
}


/**
* deletes files in a project
*
* files are not deleted finally but are backuped.
*
* @param integer project_id
* @param array files to delete
* @param integer timestamp of the deletedate
* @return void
*/
function tmpdeletefiles($project_id,$deletefiles,$deletedate)
{
	global $db;
	$db->query("UPDATE ".$db->getPrefix()."files SET deletedate=$deletedate WHERE project_id='$project_id' AND file_id IN('".implode("','",$deletefiles)."')");
	$db->query("UPDATE ".$db->getPrefix()."versions SET deletedate=$deletedate WHERE project_id='$project_id' AND file_id IN('".implode("','",$deletefiles)."')");
}


/**
* deletes versions in a project
*
* versions are not deleted finally but are backuped.
*
* @param integer project_id
* @param array versions to delete
* @param integer timestamp of the deletedate
* @return void
*/
function tmpdeleteversion($project_id,$deleteversion,$deletedate)
{
	global $db;
	$file_info = $db->query_first("SELECT file_id FROM ".$db->getPrefix()."versions WHERE project_id='$project_id' AND version_id=$deleteversion",MYSQL_ASSOC);
	$db->query("UPDATE ".$db->getPrefix()."versions SET deletedate=$deletedate WHERE project_id='$project_id' AND version_id=$deleteversion");
	$update_info = $db->query_first("SELECT version_id FROM ".$db->getPrefix()."versions WHERE project_id='$project_id' AND file_id='$file_info[file_id]' AND deletedate=0 ORDER BY createdate DESC LIMIT 1");
	if(!$update_info['version_id'])
	{
		$db->query("UPDATE ".$db->getPrefix()."files SET deletedate=$deletedate WHERE project_id='$project_id' AND file_id='$file_info[file_id]'");
	}
	else
	{
		$db->query("UPDATE ".$db->getPrefix()."files SET version_id='$update_info[version_id]' WHERE project_id='$project_id' AND file_id='$file_info[file_id]'");
	}
}



/**
* delete all backups of an project
*
* @param integer project_id
* @return void
*/
function deletebackups($project_id)
{
	global $db;
	
	$result = $db->query("SELECT v.version_id,f.extension FROM ".$db->getPrefix()."versions v LEFT JOIN ".$db->getPrefix()."files f ON v.file_id=f.file_id WHERE v.project_id='$project_id' AND v.deletedate<>'0'");
	while($row = $db->fetch_array($result,MYSQL_ASSOC))
	{
		@unlink("./files/".$row[version_id].ifelse($row['extension'],".".$row['extension']));
	}
	$db->query("DELETE FROM ".$db->getPrefix()."files WHERE project_id=$project_id AND deletedate<>'0'");
	$db->query("DELETE FROM ".$db->getPrefix()."versions WHERE project_id=$project_id AND deletedate<>'0'");
}











/**
* adds a new file/version to the project
*
* @param integer project_id
* @param string filename (without extension)
* @param string fileextension
* @param string directory of the file
* @param string comment on this file
* @param string log on this version
* @param integer timestamp of the createdate
* @param string optionally checksum of the new file
* @param integer optionally the size (in bytes) of the new file
* @return array version_id and file_id
*/
function addfile($project_id,$filename,$extension,$dir,$comment,$log,$createdate,$checksum="",$size=0)
{
		global $db,$user_info;
		
		
		$old_version = $db->query_first("SELECT f.file_id,f.filename,f.extension,f.dir,v.version_id,v.version,v.checksum 
		FROM ".$db->getPrefix()."files f LEFT JOIN ".$db->getPrefix()."versions v ON f.version_id=v.version_id 
		WHERE (f.project_id='$project_id' AND f.filename='".addslashes($filename)."' AND f.extension='".addslashes($extension)."' AND f.dir='".addslashes($dir)."')",MYSQL_ASSOC);
		if(is_array($old_version))
		{
			$newfile = 0;
			$file_id = $old_version['file_id'];
			$newversion = $old_version['version']+0.1;
		}
		else
		{
			if($dir!="") $db->query("INSERT IGNORE INTO ".$db->getPrefix()."files (project_id,dir,createdate,isfile) VALUES ('$project_id','$dir','$createdate','0')");
			$db->query("INSERT INTO ".$db->getPrefix()."files (project_id,filename,extension,dir,createdate,comment,isfile) VALUES ('$project_id','".addslashes($filename)."','".addslashes($extension)."','".addslashes($dir)."','$createdate','".addslashes($comment)."','1')");
			$newversion = 1.0;
			$file_id = $db->insert_id();
			$newfile = 1;
		}
		
		$db->query("INSERT INTO ".$db->getPrefix()."versions (version,project_id,file_id,autor_id,createdate,log,checksum,size) VALUES ('$newversion','$project_id','$file_id','$user_info[user_id]','$createdate','".addslashes($log)."','".$checksum."', '".$size."')");
		$version_id = $db->insert_id();
		$db->query("UPDATE ".$db->getPrefix()."files SET version_id='$version_id',deletedate=0 WHERE file_id='$file_id'");

		return array($version_id,$file_id);
}



/**
* updates date of last change (etc.) of a project
*
* @param integer project_id
* @param integer
* @param integer timestamp of last change
* @return void
*/
function update_project($project_id,$user_id,$time)
{
	global $db;
	$count1 = $db->query_first("SELECT count(*) FROM ".$db->getPrefix()."files WHERE project_id=$project_id AND isfile=1 AND deletedate=0",MYSQL_NUM);
	$files = $count1[0];
	$db->query("UPDATE ".$db->getPrefix()."projects SET lastchange='$time',lasteditor='$user_id',files='".$files."' WHERE project_id='$project_id'");
}





/**
* extracts the fileextension of a filename
*
* @param string filename
* @return string fileextension
*/
function get_fileextension($filename)
{
	$tmp = explode(".",$filename);
	if(count($tmp)>1)
	{
		$extension = $tmp[count($tmp)-1];
	}
	else
	{
		$extension = "";
	}
	unset($tmp);

	return $extension;
}


/**
* extracts the filename without fileextension of a filename
*
* @param string filename
* @return string filename without fileextension
*/
function get_filename($file)
{
	$tmp = explode(".",$file);
	if(count($tmp)>1)
	{
		$tmp[count($tmp)-1]="";
		$filename = implode(".",$tmp);
		$filename = substr($filename,0,strlen($filename)-1);
	}
	else
	{
		$filename = $file;
	}
	unset($tmp);
	return $filename;
}


/**
* returns the filename with filextension
*
* @param string filename without extension
* @param string fileextension
* @return string filename with extension
*/
function get_shownfilename($filename,$extension)
{
	return $filename.ifelse($extension,".".$extension);
}




/**
* dies with an error message
*
* @param string error template
* @return void
*/
function show_error($error_tpl="input_error", $length=-1)
{
	global $tpl,$tpls,$language,$options,$project_id, $sid, $sidviaget, $authentificationcode, $charset;
	if(!isSet($tpls['header'])) eval("\$tpls['header'] = \"".$tpl->get("header")."\";");
	if(!isSet($tpls['footer'])) eval("\$tpls['footer'] = \"".$tpl->get("footer")."\";");
	eval("\$tpl->out(\"".$tpl->get($error_tpl)."\");");
	exit;
}







/**
* show a done message
*
* @param string templatename
* @return void
*/
function show_msg($templatename="done", $length=-1, $updatetree=1)
{
	global $tpl,$tpls,$language,$options,$project_id, $sid, $sidviaget, $charset;
	if(!isSet($tpls['header'])) eval("\$tpls['header'] = \"".$tpl->get("header")."\";");
	if(!isSet($tpls['footer'])) eval("\$tpls['footer'] = \"".$tpl->get("footer")."\";");
	eval("\$tpl->out(\"".$tpl->get($templatename)."\");");
	exit;
}






/**
* sends headers to avoid caching of this site
*
* @return void
*/
function send_no_cache_header()
{
	header ("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	header ("Last-Modified: " . gmdate ("D, d M Y H:i:s") . " GMT");
	header ("Cache-Control: no-cache, must-revalidate");
	header ("Pragma: no-cache");
}





/**
* sends notification emails
*
* @param integer project_id
* @param integer
* @param string
* @param string
* @param integer timestamp
* @param array
* @return void
*/
function notify($project_id,$updaterid,$updater,$senderemail,$time,$type,$updates,$log="")
{
	global $db,$language,$project_info,$_SERVER;
	$r_user = $db->query("SELECT u.displayname,u.email FROM ".$db->getPrefix()."user u LEFT JOIN ".$db->getPrefix()."notify n USING(user_id) WHERE u.user_id<>'$updaterid' AND project_id=$project_id");
	//$r_user = $db->query("SELECT u.displayname,u.email FROM ".$db->getPrefix()."user u LEFT JOIN ".$db->getPrefix()."notify n USING(user_id) WHERE project_id=$project_id");
	//$r_user = $db->query("SELECT u.displayname,u.email FROM ".$db->getPrefix()."user u LEFT JOIN ".$db->getPrefix()."notify n USING(user_id) WHERE project_id=$project_id AND u.user_id='$updaterid'");
	
	
	$url = "http://".$_SERVER['SERVER_NAME'].dirname($_SERVER['REQUEST_URI']);
	if(substr($url,-1)!="/") $url.="/";
	$url .= "index.php";
	
	switch($type)
	{
		case "newfolder":
			
			while($r_user1 = $db->fetch_array($r_user,MYSQL_ASSOC))
			{
				// replace variables
				$mail = $language['mail_newfolder'];
				$mail = str_replace("{toname}",$r_user1['displayname'],$mail);
				$mail = str_replace("{updater}",$updater,$mail);
				$mail = str_replace("{project_name}",$project_info['project_name'],$mail);
				$mail = str_replace("{dirname}",implode("\n", $updates),$mail);
				$mail = str_replace("{url}",$url,$mail);
				$mail = str_replace("{date}",date("j.m.Y - H:i",$time),$mail);
				$betreff = $language['mail_newfolder_subject'];
				$betreff = str_replace("{project_name}",$project_info['project_name'],$betreff);

				// send mail
				@mail("$r_user1[email]","$betreff", "$mail", "From: $updater\r\nReply-To: $senderemail");
			}
			
			break;
		
		case "newfile":
			
			while($r_user1 = $db->fetch_array($r_user,MYSQL_ASSOC))
			{
				if(is_array($updates)) sort($updates);
				// replace variables
				$mail = $language['mail_newfile'];
				$mail = str_replace("{toname}",$r_user1['displayname'],$mail);
				$mail = str_replace("{updater}",$updater,$mail);
				$mail = str_replace("{project_name}",$project_info['project_name'],$mail);
				$mail = str_replace("{newfile}",implode("\n", $updates),$mail);
				$mail = str_replace("{date}",date("j.m.Y - H:i",$time),$mail);
				$mail = str_replace("{url}",$url,$mail);
				$mail = str_replace("{log}",$log,$mail);
				$betreff = $language['mail_newfile_subject'];
				$betreff = str_replace("{project_name}",$project_info['project_name'],$betreff);
				// send mail
				@mail("$r_user1[email]","$betreff", "$mail", "From: $updater\r\nReply-To: $senderemail");
			}
			
			break;
			
			
		case "delete":
		
			while($r_user1 = $db->fetch_array($r_user,MYSQL_ASSOC))
			{
				if(is_array($updates)) sort($updates);
				// replace variables
				$mail = $language['mail_delete'];
				$mail = str_replace("{toname}",$r_user1['displayname'],$mail);
				$mail = str_replace("{updater}",$updater,$mail);
				$mail = str_replace("{project_name}",$project_info['project_name'],$mail);
				$mail = str_replace("{delete}",implode("\n", $updates),$mail);
				$mail = str_replace("{date}",date("j.m.Y - H:i",$time),$mail);
				$mail = str_replace("{url}",$url,$mail);
				$betreff = $language['mail_delete_subject'];
				$betreff = str_replace("{project_name}",$project_info['project_name'],$betreff);
				
				// send mail
				@mail("$r_user1[email]","$betreff", "$mail", "From: $updater\r\nReply-To: $senderemail");
			}
			
			break;

		case "deleteversion":
		
			while($r_user1 = $db->fetch_array($r_user,MYSQL_ASSOC))
			{
				if(is_array($updates)) sort($updates);
				// replace variables
				$mail = $language['mail_deleteversion'];
				$mail = str_replace("{toname}",$r_user1['displayname'],$mail);
				$mail = str_replace("{updater}",$updater,$mail);
				$mail = str_replace("{project_name}",$project_info['project_name'],$mail);
				$mail = str_replace("{delete}",implode("\n", $updates),$mail);
				$mail = str_replace("{date}",date("j.m.Y - H:i",$time),$mail);
				$mail = str_replace("{url}",$url,$mail);
				$betreff = $language['mail_deleteversion_subject'];
				$betreff = str_replace("{project_name}",$project_info['project_name'],$betreff);
				
				// send mail
				@mail("$r_user1[email]","$betreff", "$mail", "From: $updater\r\nReply-To: $senderemail");
			}
			
			break;

			
		case "newnotice":
			
			while($r_user1 = $db->fetch_array($r_user,MYSQL_ASSOC))
			{
				// replace variables
				$mail = $language['mail_newnotice'];
				$mail = str_replace("{toname}",$r_user1['displayname'],$mail);
				$mail = str_replace("{updater}",$updater,$mail);
				$mail = str_replace("{project_name}",$project_info['project_name'],$mail);
				$mail = str_replace("{notice}",implode("\n\n", $updates),$mail);
				$mail = str_replace("{url}",$url,$mail);
				$mail = str_replace("{date}",date("j.m.Y - H:i",$time),$mail);
				$betreff = $language['mail_newnotice_subject'];
				$betreff = str_replace("{project_name}",$project_info['project_name'],$betreff);

				// send mail
				@mail("$r_user1[email]","$betreff", "$mail", "From: $updater\r\nReply-To: $senderemail");
			}
			
			break;

	}
}



/**
* select the checksum of a version
*
* @param integer version_id
* @return string checksum
*/
function getchecksum($version_id)
{
	global $db;
	
	$r_version = $db->query_first("SELECT v.version_id,v.checksum,f.extension FROM ".$db->getPrefix()."versions v 
	LEFT JOIN ".$db->getPrefix()."files f ON f.file_id=v.file_id WHERE v.version_id='$version_id'");
	$filename="./files/$version_id".ifelse($r_version['extension'],".".$r_version['extension']);

	/* Checksumme noch nicht gespeichert => aus datei lesen */
	if(!$r_version['checksum'])
	{
		return checksum_file($filename);
	}
	
	/* Checksumme aus DB zurckgeben */
	else
	{
		return $r_version['checksum'];
	}
}


/**
* calculates the checksum of a file
*
* @param string filename
* @return string checksum
*/
function checksum_file($filename)
{
	if(file_exists($filename))
	{
		if(function_exists("md5_file"))
		{
			return md5_file($filename);
		}
		else
		{
			$fp=fopen($filename,"rb");
			$contents=fread($fp,filesize($filename));
			fclose($fp);
			return calc_checksum($contents);
		}
	}
	else return false;
}



/**
* calculates the checksum of a string
*
* @param string data
* @return string checksum
*/
function calc_checksum($data)
{
	return md5($data);	
}





/**
* creates the javascript tree menu
* 
* @param string="" startdirectory
* @return string treedata
*/
function maketreedata($startdir="")
{
	global $tree_dirs, $tree_files, $project_info, $nodeid, $sidviaget;
	
	$treedata="";
	$dirs=array();
	$files=array();

	if(!$startdir)
	{
		$treedata.="var TREE_ITEMS = [\n['".htmlconverter($project_info['project_name'])."','browse.php?project_id=$project_info[project_id]&amp;dir=&amp;sid=$sidviaget',\n";
	}
	if(isset($tree_dirs[$startdir."/"])) $dirs=$tree_dirs[$startdir."/"];
	if(isset($tree_files[$startdir."/"])) $files=$tree_files[$startdir."/"];
	
	$i=0;
	if(count($dirs))
	{	
		foreach($dirs as $row)
		{
			if($row['dir'])
			{
				if($i>0 || $startdir) $treedata.=",\n";
				$treedata.="['".htmlconverter(substr(strrchr($row['dir'],"/"),1))."','browse.php?project_id=$project_info[project_id]&amp;dir=$row[dir]&amp;sid=$sidviaget','1'";
				$treedata.=maketreedata($row['dir']);
				//$treedata.="\n]";
				$i++;
			}
		}
	}
	
	if(count($files))
	{
		foreach($files as $row)
		{
			if($i>0 || $startdir) $treedata.=",\n";
			//$treedata.="tree.add($nodeid,$pid,'".get_shownfilename($row['filename'],$row['extension'])."','browse.php?project_id=$project_info[project_id]&amp;dir=$row[dir]&amp;file_id=$row[file_id]&amp;version_id=$row[version_id]&amp;sid=$sidviaget');\n";
			$treedata.="['".htmlconverter(get_shownfilename($row['filename'],$row['extension']))."','browse.php?project_id=$project_info[project_id]&amp;dir=$row[dir]&amp;file_id=$row[file_id]&amp;version_id=$row[version_id]&amp;sid=$sidviaget','0']\n";
			$i++;
		}
	}
	if($startdir) $treedata.="\n]\n";

	return $treedata;
}






/**
* alias to the htmlentities function 
*
* @param string text
* @return string text
*/
function htmlconverter($text)
{
	return htmlentities($text); 	
}





/**
* inverse function to htmlconverter() 
* 
* @param string text
* @return string text
*/
function rehtmlconverter($text)
{
	$trans_tbl = get_html_translation_table (HTML_ENTITIES);
	$trans_tbl = array_flip ($trans_tbl);
	return strtr ($text, $trans_tbl);
}





/**
* applies intval on all items of an array
*
* @param array
* @return array
*/
function intval_array(&$array)
{
	reset($array);
	while(list($key,$val)=each($array))
	{
		if(is_array($val)) $array[$key]=intval_array($val);
		else $array[$key]=intval($val);
	}
	return $array;
}

?>